/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef __CINSTRUCTIONFACTORY_H__
#define __CINSTRUCTIONFACTORY_H__

#include "Iinstr.hpp"

#include <string.h>

#include <iostream>
#include <sstream>
#include <map>
#include <vector>
#include <algorithm>

#include <dlfcn.h>
#include <errno.h>

#include <dirent.h>

/**
 * Name:    CInstructionFactory
 * Purpose: Factory for creating commands from shared libraries
 */
class CInstrFactory {
private:
	std::map<void*, Iinstr*> m_libStore;
	std::vector<std::string> m_files;

	Iinstr* loadFromLib(std::string lib);
public:
	Iinstr* getNextInstr();
	size_t getNumFiles();
	size_t searchLibsInDir(std::string dir);
	void loadLibsIntoMap(map<short, Iinstr*>& instr, string dir);
	CInstrFactory() : m_libStore(), m_files() {};
	~CInstrFactory();
};

#endif
