/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef __CMEM_H__
#define __CMEM_H__

#include "cdat.hpp"
#include <vector>
#include <stdexcept>


typedef int MEMORY_ADDRESS;


/**
 * Name:    CMem
 * Purpose: Class representing the memory of our emulated machine
 */
template <typename T>
class CMem
{
private:
	//MAX_MEMORY-1 zugreifbare Speicherzellen
	const int MAX_MEMORY;
	std::vector<T> m_memory;
public:
	/* aligns to BIT_LEN words, aka. does calc direct memorycell from address */
	void set(const MEMORY_ADDRESS address, const T data);
	T get(const MEMORY_ADDRESS address) const;

	/* doesn't align, user has to do */
	void setDirect(const MEMORY_ADDRESS address, const T data);
	T getDirect(const MEMORY_ADDRESS address) const;
	CMem(int size) : MAX_MEMORY(size), m_memory(size,(T)NULL) {};
};

#endif
