/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef __CPMEM_H__
#define __CPMEM_H__

template <typename T>
class CPMem;

#include "cdat.hpp"
#include <vector>
#include <stdexcept>
#include <stdlib.h>

#include "Iinstr.hpp"

typedef int MEMORY_ADDRESS;

/**
 * Name:    CMem
 * Purpose: Class representing the memory of our emulated machine
 */
template <typename T>
class CPMem
{
private:
	//MAX_MEMORY-1 zugreifbare Speicherzellen aus BYTE_COUNT Bytes
	//adressierung erfolgt aber byteweise
	const int MAX_MEMORY;
	std::vector<T> m_memory;
public:
	//wert aus referenz auslesen und in vetor speichern (index zugriff!)
	//address 0 ist ProgramCounter
	void set(const MEMORY_ADDRESS address, const T data);
	//retuniert referenz eines cdat objekts mit dem Wert von address
	T get(const MEMORY_ADDRESS address) const;
	T getByte(const MEMORY_ADDRESS address) const;
	CPMem(int size) : MAX_MEMORY(size), m_memory(size, (T)NULL) {};
	~CPMem();
};

#endif
