/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <boost/lexical_cast.hpp>
#include <boost/dynamic_bitset.hpp>
#include "disasm.h"

using namespace boost;

Iinstr* disasm::decode(std::string str)
{
	/* we need 0x prefix */
	string hex = "0x";
	hex.append(str);

	CDat val =  lexical_cast<uint32_from_hex>(hex);
	return this->decodeNum(val);
}


Iinstr* disasm::decodeNum(CDat val)
{
	dynamic_bitset<> bits(32,val), opcode(32,val), condition(9), args(32);

	args = opcode;
	args.resize(23);

	opcode >>= 23;
	condition = opcode;
	opcode.resize(5);

	condition >>= 5;
	condition.resize(4);

	//cout << "<" << hex << "> is in int  " << val << "\t is binary " << bits << " opcode?" << opcode << " condition " << condition << endl;
	try {
		Iinstr* instr = decodeOpcode(opcode.to_ulong());
		instr->decodeCondition(condition.to_ulong());
		instr->loadBits(args);
		instr->evalInstr();
		return instr;
	}
	catch(std::string &e) {
		cerr << " Error: " << e << endl;
	}
	return NULL;
}

std::string disasm::decodeToString(std::string str)
{
	return this->decode(str)->toString();
}

Iinstr* disasm::decodeOpcode(short opcode)
{
	auto iter = instrs.find(opcode);
	if(iter != instrs.end()) {
		Iinstr* p = (iter->second)->getNew();
		return p;
	}
	else {
		stringstream err;
		err << "opcode not found. in was " << opcode  << endl;
		throw err.str();
	}
}
