/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"

class Caddi : public Iinstr {
	public:
		Caddi();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Caddi();
}

Iinstr* Caddi::getNew() {
	return new Caddi();
}


/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Caddi::Caddi()
{
	opcode = B5(00010);
	name = "addi";
}

void Caddi::evalInstr()
{
	this->m_s = argbits[2];
	this->m_c = argbits[1];
	this->m_d = argbits[0];

	argbits >>= 3;
	dynamic_bitset<> immb = argbits;
	immb.resize(12);
	this->m_imm = this->generate12ImmSign(immb.to_ulong());

	argbits >>= 12;
	m_ra = this->getRegister(argbits);

	argbits >>= 4;
	m_rd = this->getRegister(argbits);
}

void Caddi::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat ra = this->m_cpu->getRegister(m_ra);
	CDatd reg = ra + this->m_imm;
	this->m_cpu->setRegister(m_rd, reg);
	if(!this->m_d) {
		this->m_cpu->updateFlags(reg, ra, this->m_imm);
	}
}

std::string Caddi::toString()
{
	stringstream op;
	op << this->getName();

	if(m_d) op << 'D';
	if(m_s) op << 'S';
	if(m_c) op << 'C';

	op << this->getConditionFlag() << " r" << m_rd << ", r" << m_ra << ", " << m_imm;

	return op.str();
}
