/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"


class Cldi : public Iinstr {
	public:
		Cldi();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Cldi();
}

Iinstr* Cldi::getNew()
{
    return new Cldi();
}
/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Cldi::Cldi()
{
	opcode = B5(11010);
	name = "ldi";
}

void Cldi::evalInstr()
{
	this->m_s = argbits[2];
	this->m_hl = argbits[1];

	argbits >>= 3;
	dynamic_bitset<> immb = argbits;
	immb.resize(16);
	this->m_imm = this->generate16ImmSign(immb.to_ulong());

	argbits >>= 16;
	m_rd = this->getRegister(argbits);
}

void Cldi::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat val = this->m_cpu->getRegister(this->m_rd);
	if(this->m_s == false) {
		if(this->m_hl == false) {
			val &= 0xFFFF0000;
		}
		else {
			val &= 0x0000FFFF;
		}
		this->m_cpu->setRegister(this->m_rd, val + this->m_imm);
	}
	else {
		this->m_cpu->setRegister(this->m_rd, this->m_imm);
	}
}

std::string Cldi::toString()
{
	stringstream op;
	op << this->getName();

	if(m_hl) op << 'H';
	if(m_s) op << 'S';

	op << this->getConditionFlag() << " r" << m_rd << ", " << m_imm;

	return op.str();
}
