/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"


class Cldx : public Iinstr {
	public:
		Cldx();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Cldx();
}

Iinstr* Cldx::getNew()
{
    return new Cldx();
}

/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Cldx::Cldx()
{
	opcode = B5(10100);
	name = "ldx";
}

void Cldx::evalInstr()
{
	this->m_s = true;

	dynamic_bitset<> immb = argbits;
	immb.resize(15);
	this->m_imm = this->generate15ImmSign(immb.to_ulong());

	argbits >>= 15;
	m_ra = this->getRegister(argbits);
	argbits >>= 4;
	m_rd = this->getRegister(argbits);

}

void Cldx::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat val = this->m_cpu->getRegister(this->m_ra);
	val += m_imm;
	this->m_cpu->setRegister(this->m_rd, this->m_cpu->getProg(val)->toNum());
}

std::string Cldx::toString()
{
	stringstream op;
	op << this->getName();

	op << this->getConditionFlag() << " r" << m_rd << ", " << m_imm << "(r" <<  m_ra << ")";

	return op.str();
}
