/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"

class Cmovf : public Iinstr {
	private:
		bool m_type;
	public:
		Cmovf();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Cmovf();
}

Iinstr* Cmovf::getNew() {
	return new Cmovf();
}

/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Cmovf::Cmovf() : m_type(0)
{
	opcode = B5(01100);
	name = "movf";
}

void Cmovf::evalInstr()
{
	this->m_type = argbits[18];
	if(this->m_type) {
		name = "movpf";
	}
	else {
		name = "movsf";
	}
	
	argbits >>= 19;
	m_rd = this->getRegister(argbits);
}

void Cmovf::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat val;
	if(this->m_type) {
		val = this->m_cpu->getFlags();
	}
	else {
		val = this->m_cpu->getStack();
	}
	this->m_cpu->setRegister(this->m_rd, val);
}

std::string Cmovf::toString()
{
	stringstream op;
	op << this->getName();
	op << " r" << m_rd;
	return op.str();
}
