/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"


class Corx : public Iinstr {
	public:
		Corx();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Corx();
}

Iinstr* Corx::getNew()
{
    return new Corx();
}
/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Corx::Corx()
{
	opcode = B5(00111);
	name = "orx";
}

void Corx::evalInstr()
{
	this->m_d = argbits[0];
	this->m_f = argbits[1];
	this->m_hl = argbits[2];

	argbits >>= 3;
	dynamic_bitset<> immb = argbits;
	immb.resize(16);
	this->m_imm = this->generate16ImmSign(immb.to_ulong());

	argbits >>= 16;
	m_rd = this->getRegister(argbits);
}

void Corx::execInstr()
{
	CDat val = this->m_cpu->getRegister(this->m_rd);
	CDat imm = generate16ImmFill(this->m_imm);
	val |= imm;

	this->m_cpu->setRegister(this->m_rd, val);
	if(!this->m_d) {
		this->m_cpu->updateFlags(val);
	}
}

std::string Corx::toString()
{
	stringstream op;
	op << this->getName();

	if(m_hl) op << 'H';
	if(m_f) op << 'F';
	if(m_d) op << 'D';

	op << this->getConditionFlag() << " r" << m_rd << ", " << m_imm;

	return op.str();
}
