/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"

class Cstackop : public Iinstr {
		char m_typ;
	public:
		Cstackop();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Cstackop();
}

Iinstr* Cstackop::getNew()
{
	return new Cstackop();
}
/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Cstackop::Cstackop() : m_typ(0)
{
	opcode = B5(01011);
	name = "stackop";
}

void Cstackop::evalInstr()
{
	argbits >>= 17;

	dynamic_bitset<> type = argbits;
	type.resize(2);
	this->m_typ = type.to_ulong();

	switch(this->m_typ) {
		case 0:
			this->name = "pop";
			break;
		case 1:
			this->name = "disc";
			break;
		case 2:
			this->name = "fetch";
			break;
		case 3:
			this->name = "push";
			break;
		default:
			cerr << "What have you done? 2 bits that have more than 4 values?!" << endl;
	}

	argbits >>= 2;
	m_rd = this->getRegister(argbits);
}

void Cstackop::execInstr()
{
	CDat sp = this->m_cpu->getStack();

	switch(this->m_typ) {
		case 0:
			this->m_cpu->setRegister(m_rd, this->m_cpu->getRAM(sp));
			sp += 4;
			break;
		case 1:
			sp += 4;
			break;
		case 2:
			this->m_cpu->setRegister(m_rd, this->m_cpu->getRAM(sp));
			break;
		case 3:
			sp -= 4;
			this->m_cpu->setRAM(sp, this->m_cpu->getRegister(m_rd));
			break;
		default:
			cerr << "What have you done? 2 bits that have more than 4 values?!" << endl;
	}

	this->m_cpu->setStack(sp);
}

std::string Cstackop::toString()
{
	stringstream op;
	op << this->getName();

	op << this->getConditionFlag();
	op << " r" << m_rd;
	return op.str();
}
