/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"


class Cstb : public Iinstr {
	public:
		Cstb();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Cstb();
}

Iinstr* Cstb::getNew()
{
    return new Cstb();
}

/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Cstb::Cstb()
{
	opcode = B5(10011);
	name = "stb";
}

void Cstb::evalInstr()
{
	this->m_s = true;

	dynamic_bitset<> immb = argbits;
	immb.resize(15);
	this->m_imm = this->generate15ImmSign(immb.to_ulong());

	argbits >>= 15;
	m_ra = this->getRegister(argbits);
	argbits >>= 4;
	m_rd = this->getRegister(argbits);

}

#include <iomanip>
#define AS_HEX std::hex << setw(8) << setfill('0')

void Cstb::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat addr = this->m_cpu->getRegister(this->m_ra);
	addr += m_imm;
	MEMORY_ADDRESS temp = (addr & (~(BYTE_COUNT-1)));
	CDat offset = addr-temp;
	CDat val = this->m_cpu->getRAM(temp);

	val = val & ~( ((1<<8)-1) << (offset*8));

	//data to insert
	CDat source = this->m_cpu->getRegister(this->m_rd);
	source = (source & (((1<<8)-1))) << (offset*8);

	val += source;

	this->m_cpu->setRAM(temp, val);
}

std::string Cstb::toString()
{
	stringstream op;
	op << this->getName();

	op << this->getConditionFlag() << " r" << m_rd << ", " << m_imm << "(r" <<  m_ra << ")";

	return op.str();
}
