/*   `Deep Thought', a softcore CPU implemented on a FPGA

    Copyright (C) 2010 Markus Hofstaetter <markus.manrow@gmx.at>
    Copyright (C) 2010 Martin Perner <e0725782@student.tuwien.ac.at>
    Copyright (C) 2010 Stefan Rebernig <stefan.rebernig@gmail.com>
    Copyright (C) 2010 Manfred Schwarz <e0725898@student.tuwien.ac.at>
    Copyright (C) 2010 Bernhard Urban <lewurm@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include "../Iinstr.hpp"

class Csub : public Iinstr {
	public:
		Csub();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Csub();
}

Iinstr* Csub::getNew() {
	return new Csub();
}

/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Csub::Csub()
{
	opcode = B5(00001);
	name = "sub";
}

void Csub::evalInstr()
{
	this->m_d = argbits[0];
	this->m_c = argbits[1];
	
	argbits >>= 11;
	m_rb = this->getRegister(argbits);

	argbits >>= 4;
	m_ra = this->getRegister(argbits);

	argbits >>= 4;
	m_rd = this->getRegister(argbits);
}

void Csub::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat ra = this->m_cpu->getRegister(m_ra);
	CDat rb = this->m_cpu->getRegister(m_rb);
	CDatd val = ra - rb;
	this->m_cpu->setRegister(m_rd, val);
	if(!this->m_d) {
		this->m_cpu->updateFlags(val, ra, (~rb)+1);
	}
}

std::string Csub::toString()
{
	stringstream op;
	op << this->getName();

	if(m_d) op << 'D';
	if(m_c) op << 'C';

	op << " r" << m_rd << ", r" << m_ra << ", r" << m_rb;
	return op.str();
}
